@@ -20,7 +20,7 @@ package MongoDB;
 # ABSTRACT: Official MongoDB Driver for Perl
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 # regexp_pattern was unavailable before 5.10, had to be exported to load the
 # function implementation on 5.10, and was automatically available in 5.10.1
@@ -83,11 +83,11 @@ MongoDB - Official MongoDB Driver for Perl
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
-This is the Alpha 1 release for v2.0.0.
+This is the Beta 1 release for v2.0.0.
 
-=head1 ALPHA RELEASE
+=head1 BETA RELEASE
 
 B<NOTE:> The v1.999.x versions are development releases in advance of the
 MongoDB Perl Driver v2.0.0. They are available for evaluation and testing
@@ -96,32 +96,25 @@ and should not be used in production.
 Details on incompatible changes and how to upgrade applications may be
 found in L<MongoDB::Upgrading>.
 
-This development release includes several new features, including:
+This development release includes several significant changes since Alpha
+1, including:
 
 =over
 
 =item *
 
-BSON encoding and decoding uses the external L<BSON> and optional L<BSON::XS>
-libraries
+The API for multi-document ACID transactions is now available.  See
+L<MongoDB::ClientSession> for details. (Requires MongoDB 4.0)
 
 =item *
 
-Support for MongoDB 3.6 features, including sessions and retryable writes
-
-=item *
-
-Wire protocol compression
-
-=item *
-
-Command monitoring callback support
+Change streams via the C<watch> method are now available at both single
+databases and client (all databases) levels. (Requires MongoDB 4.0)
 
 =back
 
-B<NOTE:> Some MongoDB 4.0 features are still under development and are not
-yet included in the driver, including transactions and
-database/cluster-wide change streams.
+As of the Beta release, the driver is feature complete for MongoDB 4.0
+support.
 
 =head1 SYNOPSIS
 
