@@ -19,7 +19,7 @@ package MongoDB::BulkWrite;
 # ABSTRACT: MongoDB bulk write interface
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use MongoDB::Error;
 use MongoDB::Op::_BulkWrite;
@@ -265,7 +265,7 @@ sub execute {
 
     $write_concern ||= $self->collection->write_concern;
 
-    my $session = $self->collection->_get_session_from_hashref( $options );
+    my $session = $self->_client->_get_session_from_hashref( $options );
 
     my $op = MongoDB::Op::_BulkWrite->_new(
         client                   => $self->_client,
@@ -300,7 +300,7 @@ MongoDB::BulkWrite - MongoDB bulk write interface
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
 =head1 SYNOPSIS
 
