@@ -19,7 +19,7 @@ package MongoDB::CommandResult;
 # ABSTRACT: MongoDB generic command result document
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -115,6 +115,18 @@ sub last_wtimeout {
         || exists $self->output->{writeConcernError} );
 }
 
+#pod =method last_error_labels
+#pod
+#pod Returns an array of error labels from the command, or an empty array if there
+#pod are none
+#pod
+#pod =cut
+
+sub last_error_labels {
+    my ( $self ) = @_;
+    return $self->output->{errorLabels} || [];
+}
+
 #pod =method assert
 #pod
 #pod Throws an exception if the command failed.
@@ -159,7 +171,7 @@ MongoDB::CommandResult - MongoDB generic command result document
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
 =head1 DESCRIPTION
 
@@ -190,6 +202,11 @@ Error string (if any) or the empty string if there was no error.
 
 True if a write concern error or timeout occurred or false otherwise.
 
+=head2 last_error_labels
+
+Returns an array of error labels from the command, or an empty array if there
+are none
+
 =head2 assert
 
 Throws an exception if the command failed.
