@@ -19,7 +19,7 @@ package MongoDB::GridFSBucket;
 # ABSTRACT: A file storage abstraction
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo;
 use MongoDB::GridFSBucket::DownloadStream;
@@ -163,6 +163,18 @@ has max_time_ms => (
     required => 1,
 );
 
+#pod =attr disable_md5
+#pod
+#pod When true, files will not include the deprecated C<md5> field in the
+#pod file document.  Defaults to false.
+#pod
+#pod =cut
+
+has disable_md5 => (
+    is       => 'ro',
+    isa      => Boolish,
+);
+
 # determines whether or not to attempt index creation
 has _tried_indexing => (
     is => 'rwp',
@@ -573,7 +585,7 @@ MongoDB::GridFSBucket - A file storage abstraction
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
 =head1 SYNOPSIS
 
@@ -604,7 +616,7 @@ object.
 =head2 Data model
 
 A GridFS file is represented in MongoDB as a "file document" with
-information like the file's name, length, MD5 hash, and any user-supplied
+information like the file's name, length, and any user-supplied
 metadata.  The actual contents are stored as a number of "chunks" of binary
 data.  (Think of the file document as a directory entry and the chunks like
 blocks on disk.)
@@ -631,10 +643,6 @@ uploadDate – the date and time this file was added to GridFS, stored as a BSON
 
 =item *
 
-md5 – a hash of the contents of the stored file
-
-=item *
-
 filename – the name of this stored file; the combination of filename and uploadDate (millisecond resolution) must be unique
 
 =item *
@@ -643,6 +651,10 @@ metadata – any additional application data the user wishes to store (optional)
 
 =item *
 
+md5 – DEPRECATED a hash of the contents of the stored file (store this in C<metadata> if you need it) (optional)
+
+=item *
+
 contentType – DEPRECATED (store this in C<metadata> if you need it) (optional)
 
 =item *
@@ -735,6 +747,11 @@ L<MongoDB::Database> object.
 B<Note>: this will only be used for server versions 2.6 or greater, as that
 was when the C<$maxTimeMS> meta-operator was introduced.
 
+=head2 disable_md5
+
+When true, files will not include the deprecated C<md5> field in the
+file document.  Defaults to false.
+
 =head1 METHODS
 
 =head2 find
