@@ -19,7 +19,7 @@ package MongoDB::GridFSBucket::DownloadStream;
 # ABSTRACT: File handle abstraction for downloading
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo;
 use Types::Standard qw(
@@ -50,9 +50,9 @@ use namespace::clean -except => 'meta';
 #pod * uploadDate – the date and time this file was added to GridFS, stored as a
 #pod   BSON datetime value and inflated per the bucket's
 #pod   L<bson_codec|MongoDB::GridFSBucket/bson_codec> attribute.
-#pod * md5 – a hash of the contents of the stored file
 #pod * filename – the name of this stored file; this does not need to be unique
 #pod * metadata – any additional application-specific data
+#pod * md5 – DEPRECATED
 #pod * contentType – DEPRECATED
 #pod * aliases – DEPRECATED
 #pod
@@ -437,7 +437,7 @@ MongoDB::GridFSBucket::DownloadStream - File handle abstraction for downloading
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
 =head1 SYNOPSIS
 
@@ -487,10 +487,6 @@ uploadDate – the date and time this file was added to GridFS, stored as a BSON
 
 =item *
 
-md5 – a hash of the contents of the stored file
-
-=item *
-
 filename – the name of this stored file; this does not need to be unique
 
 =item *
@@ -499,6 +495,10 @@ metadata – any additional application-specific data
 
 =item *
 
+md5 – DEPRECATED
+
+=item *
+
 contentType – DEPRECATED
 
 =item *
