@@ -19,7 +19,7 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo;
 use MongoDB::Cursor;
@@ -35,13 +35,13 @@ use MongoDB::_Topology;
 use MongoDB::_Constants;
 use MongoDB::_Credential;
 use MongoDB::_URI;
-use BSON 1.006003;
+use BSON 1.006005;
 use Digest::MD5;
 use UUID::URandom;
 use Tie::IxHash;
 use Time::HiRes qw/usleep/;
 use Carp 'carp', 'croak', 'confess';
-use Safe::Isa;
+use Safe::Isa 1.000007;
 use Scalar::Util qw/reftype weaken/;
 use boolean;
 use Encode;
@@ -1075,7 +1075,8 @@ has _write_concern => (
 sub _build__write_concern {
     my ($self) = @_;
     return MongoDB::WriteConcern->new(
-        ( $self->w        ? ( w        => $self->w )        : () ),
+        # Must check for defined as w can be 0, and defaults to undef
+        ( defined $self->w ? ( w        => $self->w )        : () ),
         ( $self->wtimeout ? ( wtimeout => $self->wtimeout ) : () ),
         ( $self->j        ? ( j        => $self->j )        : () ),
     );
@@ -1105,28 +1106,6 @@ sub _build__read_concern {
     );
 }
 
-has _cluster_time => (
-    is => 'rwp',
-    isa => Maybe[Document],
-    init_arg => undef,
-    default => undef,
-);
-
-sub _update_cluster_time {
-    my ( $self, $cluster_time ) = @_;
-
-    # Only update the cluster time if it is more recent than the current entry
-    if ( ! defined $self->_cluster_time ) {
-        $self->_set__cluster_time( $cluster_time );
-    } else {
-        if ( $cluster_time->{'clusterTime'}
-           > $self->_cluster_time->{'clusterTime'} ) {
-            $self->_set__cluster_time( $cluster_time );
-        }
-    }
-    return;
-}
-
 #--------------------------------------------------------------------------#
 # private attributes
 #--------------------------------------------------------------------------#
@@ -1169,7 +1148,11 @@ has _topology => (
     isa      => InstanceOf ['MongoDB::_Topology'],
     init_arg => undef,
     builder  => '_build__topology',
-    handles  => { topology_type => 'type' },
+    handles  => {
+        topology_type => 'type',
+        _cluster_time => 'cluster_time',
+        _update_cluster_time => 'update_cluster_time',
+    },
     clearer  => '_clear__topology',
 );
 
@@ -1544,10 +1527,24 @@ sub send_admin_command {
     return $self->send_read_op( $op );
 }
 
+# Reset session state if we're outside an active transaction, otherwise set
+# that this transaction actually has operations
+sub _maybe_update_session_state {
+    my ( $self, $op ) = @_;
+    if ( defined $op->session && ! $op->session->_active_transaction ) {
+        $op->session->_set__transaction_state( TXN_NONE );
+    } elsif ( defined $op->session ) {
+        $op->session->_set__has_transaction_operations( 1 );
+    }
+}
+
 # op dispatcher written in highly optimized style
 sub send_direct_op {
     my ( $self, $op, $address ) = @_;
     my ( $link, $result );
+
+    $self->_maybe_update_session_state( $op );
+
     ( $link = $self->{_topology}->get_specific_link($address) ), (
         eval { ($result) = $op->execute($link); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
@@ -1569,17 +1566,12 @@ sub send_direct_op {
 sub send_write_op {
     my ( $self, $op ) = @_;
     my ( $link, $result );
+
+    $self->_maybe_update_session_state( $op );
+
     ( $link = $self->{_topology}->get_writable_link ), (
-        eval { ($result) = $op->execute($link, $self->{_topology}->type); 1 } or do {
+        eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-            }
-            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-                $self->{_topology}->mark_stale;
-            }
-            # regardless of cleanup, rethrow the error
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
           }
       ),
@@ -1595,15 +1587,25 @@ BEGIN {
 }
 
 sub send_retryable_write_op {
-    my ( $self, $op ) = @_;
-
-    return $self->send_write_op( $op ) unless $self->retry_writes;
+    my ( $self, $op, $force ) = @_;
 
     my $result;
     my $link = $self->{_topology}->get_writable_link;
 
-    # If server doesnt support retryable writes, pretend its not enabled
-    unless ( $link->supports_retryWrites ) {
+    $self->_maybe_update_session_state( $op );
+
+    # Need to force to do a retryable write on a Transaction Commit or Abort. $force is an override for retry_writes, but theres no point trying that if the link doesnt support it anyway.
+    # This triggers on the following:
+    # * $force is not set to 'force'
+    #   (specifically for retrying writes in ending transaction operations)
+    # * retry writes is not enabled or the link doesnt support retryWrites
+    # * if an active transaction is starting or in progress
+    unless ( $link->supports_retryWrites
+        && ( $self->retry_writes || ( defined $force && $force eq 'force' ) )
+        && ( defined $op->session
+          && ! $op->session->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS )
+        )
+    ) {
         eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
@@ -1613,14 +1615,23 @@ sub send_retryable_write_op {
 
     # If we get this far and there is no session, then somethings gone really
     # wrong, so probably not worth worrying about.
-    #
-    # increment transaction id before write, but otherwise is the same for both attempts
-    $op->session->_server_session->_increment_transaction_id;
+
+    # increment transaction id before write, but otherwise is the same for both
+    # attempts. If not in a transaction, is a no-op
+    $op->session->_increment_transaction_id;
     $op->retryable_write( 1 );
 
     # attempt the op the first time
     eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
         my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+
+        # If the error is not retryable, then drop out
+        unless ( $err->$_call_if_can('_is_retryable') ) {
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+        }
+
+        # Must check if error is retryable before getting the link, in case we
+        # get a 'no writable servers' error
         my $retry_link = $self->{_topology}->get_writable_link;
 
         # Rare chance that the new link is not retryable
@@ -1670,6 +1681,17 @@ sub _try_write_op_for_link {
 sub send_read_op {
     my ( $self, $op ) = @_;
     my ( $link, $type, $result );
+
+    # Get transaction read preference if in a transaction.
+    if ( defined $op->session && $op->session->_active_transaction ) {
+        # Transactions may only read from primary in MongoDB 4.0, so get and
+        # check the read preference from the transaction settings as per
+        # transaction spec - see MongoDB::_TransactionOptions
+        $op->read_preference( $op->session->_get_transaction_read_preference );
+    }
+
+    $self->_maybe_update_session_state( $op );
+
     ( $link = $self->{_topology}->get_readable_link( $op->read_preference ) ),
       ( $type = $self->{_topology}->type ), (
         eval { ($result) = $op->execute( $link, $type ); 1 } or do {
@@ -1870,6 +1892,94 @@ sub fsync_unlock {
     return $self->send_primary_op($op);
 }
 
+sub _get_session_from_hashref {
+    my ( $self, $hashref ) = @_;
+
+    my $session = delete $hashref->{session};
+
+    if ( defined $session ) {
+        MongoDB::UsageError->throw( "Cannot use session from another client" )
+            if ( $session->client->_id ne $self->_id );
+        MongoDB::UsageError->throw( "Cannot use session which has ended" )
+            if ! defined $session->session_id;
+    } else {
+        $session = $self->_maybe_get_implicit_session;
+    }
+
+    return $session;
+}
+
+#pod =method watch
+#pod
+#pod Watches for changes on the cluster.
+#pod
+#pod Perform an aggregation with an implicit initial C<$changeStream> stage
+#pod and returns a L<MongoDB::ChangeStream> result which can be used to
+#pod iterate over the changes in the cluster. This functionality is
+#pod available since MongoDB 4.0.
+#pod
+#pod     my $stream = $client->watch();
+#pod     my $stream = $client->watch( \@pipeline );
+#pod     my $stream = $client->watch( \@pipeline, \%options );
+#pod
+#pod     while (1) {
+#pod
+#pod         # This inner loop will only run until no more changes are
+#pod         # available.
+#pod         while (my $change = $stream->next) {
+#pod             # process $change
+#pod         }
+#pod     }
+#pod
+#pod The returned stream will not block forever waiting for changes. If you
+#pod want to respond to changes over a longer time use C<maxAwaitTimeMS> and
+#pod regularly call C<next> in a loop.
+#pod
+#pod See L<MongoDB::Collection/watch> for details on usage and available
+#pod options.
+#pod
+#pod =cut
+
+sub watch {
+    my ( $self, $pipeline, $options ) = @_;
+
+    $pipeline ||= [];
+    $options ||= {};
+
+    my $session = $self->_get_session_from_hashref( $options );
+
+    return MongoDB::ChangeStream->new(
+        exists($options->{startAtOperationTime})
+            ? (start_at_operation_time => delete $options->{startAtOperationTime})
+            : (),
+        exists($options->{fullDocument})
+            ? (full_document => delete $options->{fullDocument})
+            : (full_document => 'default'),
+        exists($options->{resumeAfter})
+            ? (resume_after => delete $options->{resumeAfter})
+            : (),
+        exists($options->{maxAwaitTimeMS})
+            ? (max_await_time_ms => delete $options->{maxAwaitTimeMS})
+            : (),
+        client => $self,
+        all_changes_for_cluster => 1,
+        pipeline => $pipeline,
+        session => $session,
+        options => $options,
+        op_args => {
+            read_concern => $self->read_concern,
+            db_name => 'admin',,
+            coll_name => 1,
+            full_name => 'admin.1',
+            bson_codec => $self->bson_codec,
+            write_concern => $self->write_concern,
+            read_concern => $self->read_concern,
+            read_preference => $self->read_preference,
+            monitoring_callback => $self->monitoring_callback,
+        },
+    );
+}
+
 
 1;
 
@@ -1885,7 +1995,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v1.999.0
+version v1.999.1
 
 =head1 SYNOPSIS
 
@@ -2680,6 +2790,35 @@ The primary use of fsync is to lock the database during backup operations. This
 
 Unlocks a database server to allow writes and reverses the operation of a $conn->fsync({lock => 1}); operation.
 
+=head2 watch
+
+Watches for changes on the cluster.
+
+Perform an aggregation with an implicit initial C<$changeStream> stage
+and returns a L<MongoDB::ChangeStream> result which can be used to
+iterate over the changes in the cluster. This functionality is
+available since MongoDB 4.0.
+
+    my $stream = $client->watch();
+    my $stream = $client->watch( \@pipeline );
+    my $stream = $client->watch( \@pipeline, \%options );
+
+    while (1) {
+
+        # This inner loop will only run until no more changes are
+        # available.
+        while (my $change = $stream->next) {
+            # process $change
+        }
+    }
+
+The returned stream will not block forever waiting for changes. If you
+want to respond to changes over a longer time use C<maxAwaitTimeMS> and
+regularly call C<next> in a loop.
+
+See L<MongoDB::Collection/watch> for details on usage and available
+options.
+
 =for Pod::Coverage connected
 send_admin_command
 send_direct_op
