@@ -19,7 +19,7 @@ package MongoDB::Role::_WriteResult;
 # MongoDB interface for common write result attributes and methods
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo::Role;
 
@@ -122,4 +122,15 @@ sub last_wtimeout {
     return !!( $self->count_write_concern_errors && !$self->count_write_errors );
 }
 
+sub last_error_labels {
+    my ( $self ) = @_;
+    if ( $self->count_write_errors ) {
+        return $self->write_errors->[-1]{errorLabels} || [];
+    }
+    elsif ( $self->count_write_concern_errors ) {
+        return $self->write_errors->[-1]{errorLabels} || [];
+    }
+    return [];
+}
+
 1;
