@@ -23,7 +23,7 @@ use warnings;
 package MongoDB::_Link;
 
 use version;
-our $VERSION = 'v1.999.0'; # TRIAL
+our $VERSION = 'v1.999.1'; # TRIAL
 
 use Moo;
 use Errno qw[EINTR EPIPE];
@@ -215,6 +215,12 @@ has supports_retryWrites => (
     isa => Boolish,
 );
 
+has supports_4_0_changestreams => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
 my @connection_state_fields = qw(
     fh connected rcvbuf last_used fdset is_ssl
 );
@@ -350,6 +356,9 @@ sub set_metadata {
             : 0
         );
     }
+    if ( $self->accepts_wire_version(7) ) {
+        $self->_set_supports_4_0_changestreams(1);
+    }
 
     return;
 }
